@@ -22,6 +22,18 @@ test_that("push, peek and pop", {
     expect_error(q$pop(), "empty")
 })
 
+test_that("push, peek and pop with items", {
+    q <- Container(list(2, 3))
+    expect_equal(q$size(), 2)
+    expect_equal(q$peek(), 2)
+    expect_equal(q$pop(), 2)
+    expect_equal(q$peek(), 3)
+    expect_equal(q$pop(), 3)
+    expect_equal(q$size(), 0)
+    expect_error(q$peek(), "empty")
+    expect_error(q$pop(), "empty")
+})
+
 test_that("clear", {
     q <- Container()
     q$push("a")$push("b")$push("c")
